@@ -26,9 +26,9 @@ extern int ordp (void *);
 extern void logaddr (fbuffer *);
 extern void logchar (Char);
 extern void snapname (Char *, chbufinx, chbufinx);
-extern void wlogfl (Char *, double, int);
+extern void wlogfl (char *, double, int);
 extern void wrbufaddr (fbuffer *, int);
-extern void wrbuf (fbuffer *, int, int);
+extern void wrbuf (fbuffer *, int);
 #endif
 
 							/* the parser                         */
@@ -297,7 +297,10 @@ consoleflush (void) {
 							/* Unrecoverable errors */
 void
 fatal (int t) {
-  if (t != 0) { fprintf (errout, "\n*** dpic: "); }
+#ifdef DDEBUG
+  if (debuglevel > 0) { fprintf (log_, "\nfatal(%d)\n",t); fflush(log_); }
+#endif
+  if (t != 0) { fprintf (errout, "\n*** dpic terminating: "); }
   switch (t) {
   case 0:
 							/* blank case */
@@ -323,6 +326,9 @@ fatal (int t) {
   case 8:
     fprintf (errout, "error recovery abandoned\n");
     break;
+  case 9:
+    fprintf (errout, "malloc failed: insufficient available memory\n");
+    break;
   default:
     fprintf (errout, "unclassified fatal error\n");
     break;
@@ -351,29 +357,6 @@ openlogfile (void) {
 }
 #endif
 
-							/* Check if a file is accessible */
-int
-checkfile (Char * ifn, boolean isverbose) {
-  int cf;
-  int i = 0, j = FILENAMELEN;
-  while (j > i) {
-    if (ifn[j - 1] == ' ') { j--; } else { i = j; } }
-  if (j < FILENAMELEN) { j++; } else { fatal (1); }
-  ifn[j - 1] = '\0';
-  cf = access (ifn, 4);
-  if (!(isverbose && (cf != 0))) { return cf; }
-  fprintf (errout, " *** dpic: Searching for file \"");
-  for (i = 0; i <= (j - 2); i++) { putc (ifn[i], errout); }
-  fprintf (errout, "\" returns %d\n", cf);
-  return cf;
-}
-
-							/* Open the error stream */
-void
-openerror (void) {
-  errout = stderr;
-}
-
 							/* Printable character */
 boolean
 isprint_ (Char ch) {
@@ -388,7 +371,25 @@ wchar (FILE ** iou, Char c) {
   else if (c == crch) { fprintf (*iou, "\\r"); }
   else if (c == tabch) { fprintf (*iou, "\\t"); }
   else if (c < 32) { fprintf (*iou, "^%c", c + 64); }
-  else { fprintf (*iou, "\\%d%d%d", (c >> 6) & 7, (c >> 3) & 7, c & 7); }
+  else { fprintf (*iou, "\\%d%d%d", (c >> 6) & 3, (c >> 3) & 7, c & 7); }
+}
+
+							/* Check if a file is accessible */
+int
+checkfile (Char * ifn, boolean isverbose) {
+  int cf, i;
+  cf = access ((char *)ifn, 4);
+  if (!(isverbose && (cf != 0))) { return cf; }
+  fprintf (errout, " *** dpic: Searching for file \"");
+  for (i = 0; ifn[i] != '\0'; i++) { wchar(&errout,ifn[i]); }
+  fprintf (errout, "\" returns %d\n", cf);
+  return cf;
+}
+
+							/* Open the error stream */
+void
+openerror (void) {
+  errout = stderr;
 }
 
 /*--------------------------------------------------------------------*/
@@ -449,7 +450,7 @@ exitmacro (void) {
     else if (currentmacro->argbody->carray != NULL) {
       mbody = currentmacro->argbody;
       for (i = 1; i <= -(mbody->attrib); i++) {
-        wchar (&log_, mbody->carray[i]); } }
+        wchar (&log_,  mbody->carray[i]); } }
     putc ('\n', log_); }
 #endif
   currentmacro = NULL;
@@ -691,7 +692,7 @@ readlatex (void) {
 int
 argcount (arg * a) {
   int i = 0;
-  if (a == NULL) { markerror (805); }
+  if (a == NULL) { markerror (807); }
   while (a != NULL) {
     if (a->argbody != NULL) { i++; }
     a = a->nexta; }
@@ -719,48 +720,36 @@ arg *(findarg (arg * arlst, int k)) {
 #ifdef DDEBUG
   if (debuglevel > 0) {
     if ((i != k) || (ar == NULL)) { fprintf (log_, "not found\n"); }
-    else { wrbuf (ar->argbody, 2, 1); }
+    else { wrbuf (ar->argbody, 2); }
   }
 #endif
   return ar;
 }
 
-Char *
-trimname (Char * fn, int len) {
-  static Char fnbuf[256];
-  Char *cp = fnbuf;
-  while (--len >= 0 && *fn && !isspace ((unsigned)*fn))
-    *cp++ = *fn++;
-  *cp = 0;
-  return fnbuf;
-}
-
 							/* Start reading from file for copy "file" */
 #ifndef SAFE_MODE
 void
 pointinput (nametype * txt) {
   int i;
-  Char c = ' ';
   int FORLIM;
   if (txt == NULL) { return; }
   if (txt->segmnt == NULL) { return; }
   if (txt->len >= FILENAMELEN) { txt->len = FILENAMELEN - 1; }
   FORLIM = txt->len;
   for (i = 0; i < FORLIM; i++) { infname[i] = txt->segmnt[txt->seginx + i]; }
-  for (i = txt->len; i < FILENAMELEN; i++) { infname[i] = c; }
+  infname[txt->len] = '\0';
 #ifdef DDEBUG
   if (debuglevel > 0) {
     fprintf (log_, "Pointinput(segmnt %d, len %d) ", txt->seginx, txt->len);
     FORLIM = txt->len;
-    for (i = 0; i < FORLIM; i++) { putc (infname[i], log_); }
+    for (i = 0; i < FORLIM; i++) { wchar(&log_,infname[i]); }
     putc ('\n', log_);
     }
 #endif
   if (higherinbuf != NULL) { markerror (853); return; }
   if (checkfile (infname, true) != 0) { markerror (859); return; }
-  if (copyin != NULL) {
-       copyin = freopen (trimname (infname, sizeof (mstring)), "r", copyin); }
-  else { copyin = fopen (trimname (infname, sizeof (mstring)), "r"); }
+  if (copyin != NULL) { copyin = freopen ((char *) infname, "r", copyin); }
+  else { copyin = fopen ((char *) infname, "r"); }
   if (copyin == NULL) { fatal (1); }
   backup ();
   ch = nlch;
@@ -774,14 +763,8 @@ pointinput (nametype * txt) {
 void
 pointoutput (boolean create, nametype * txt, int *ier) {
   int i, FORLIM;
-  if (txt == NULL) {
-    *ier = 1;
-    markerror (861);
-    return; }
-  else if (txt->segmnt == NULL) {
-    *ier = 1;
-    markerror (861);
-    return; }
+  if (txt == NULL) { *ier = 1; markerror (861); return; }
+  else if (txt->segmnt == NULL) { *ier = 1; markerror (861); return; }
   *ier = 0;
   if (txt->len >= FILENAMELEN) { txt->len = FILENAMELEN - 1; }
   FORLIM = txt->len;
@@ -797,24 +780,19 @@ pointoutput (boolean create, nametype * txt, int *ier) {
     fprintf (log_, "Pointoutput(%s segmnt %d, len %d) \"",
 	     create ? " TRUE" : "FALSE", txt->seginx, txt->len);
     FORLIM = txt->len;
-    for (i = 0; i < FORLIM; i++) { putc (outfnam[i], log_); }
+    for (i = 0; i < FORLIM; i++) { wchar (&log_, outfnam[i]); }
     fprintf (log_, "\"\n");
     fflush (log_);
     }
 #endif
   if ((*ier) != 0) { markerror (861); return; }
   if (create) {
-    if (redirect != NULL) {
-      redirect= freopen (trimname (outfnam, sizeof (mstring)), "w", redirect);}
-    else {
-      redirect =  fopen (trimname (outfnam, sizeof (mstring)), "w"); }
+    if (redirect != NULL) { redirect=freopen((char *) outfnam, "w", redirect);}
+    else { redirect =  fopen ((char *)outfnam, "w"); }
     if (redirect == NULL) { markerror(876); fatal (8); }
-    return;
-    }
-  if (redirect != NULL) {
-    redirect = freopen (trimname (outfnam, sizeof (mstring)), "a", redirect); }
-  else {
-    redirect =   fopen (trimname (outfnam, sizeof (mstring)), "a"); }
+    return; }
+  if (redirect != NULL) { redirect=freopen((char *)outfnam, "a", redirect); }
+  else { redirect =   fopen ((char *)outfnam, "a"); }
   if (redirect == NULL) { markerror(876); fatal (8); }
 }
 
@@ -874,7 +852,7 @@ readstring (void) {
     putc ('\n', log_);
     fprintf (log_, " readstring done, chbufi=%d |", chbufi);
     while (j < chbufi) {
-      putc (chbuf[j], log_);
+      wchar(&log_,chbuf[j]);
       j++;
       }
     fprintf (log_, "| ");
@@ -950,7 +928,7 @@ prlex (boolean acc) {
   if (newsymb != XLaTeX) {
     fprintf (log_, " chbuf(%d:%d)=\"", oldbufi, chbufi - 1);
     FORLIM = chbufi;
-    for (i = oldbufi; i < FORLIM; i++) { putc (chbuf[i], log_); }
+    for (i = oldbufi; i < FORLIM; i++) { wchar(&log_,chbuf[i]); }
     putc ('"', log_); }
   else { fprintf (log_, "=<LaTeX>"); }
   if (newsymb == Xfloat) { wlogfl ("value", floatvalue, 0); }
@@ -987,8 +965,8 @@ copyleft (fbuffer * mac, fbuffer ** buf, int macattr) {
   if (debuglevel > 0) {
     fprintf (log_, "copyleft(%d,%d,%d):\n", odp(mac), odp(*buf), macattr); }
   if (debuglevel > 1) {
-    fprintf (log_, " input string"); wrbuf (mac, 3, 1);
-    fprintf (log_, " output"); wrbuf (*buf, 3, 1); }
+    fprintf (log_, " input string"); wrbuf (mac, 3);
+    fprintf (log_, " output"); wrbuf (*buf, 3); }
 #endif
   if (mac != NULL) { while (mac->nextb != NULL) { mac = mac->nextb; } }
   while (mac != NULL) {
@@ -1026,11 +1004,11 @@ copyleft (fbuffer * mac, fbuffer ** buf, int macattr) {
   if (copied) {
 #ifdef DDEBUG
     if (debuglevel > 0) {
-      fprintf (log_, " copyleft result\n"); wrbuf (*buf, 3, 1); }
+      fprintf (log_, " copyleft result\n"); wrbuf (*buf, 3); }
 #endif
-                            /* precede the leftmost char with a newline */
+                            /* precede the leftmost char with an ETX */
     if ((*buf)->readx <= 1) { *buf = prependbuffer (*buf); }
-    (*buf)->carray[(*buf)->readx - 1] = nlch;
+    (*buf)->carray[(*buf)->readx - 1] = etxch;
     }
 }
 
@@ -1042,8 +1020,8 @@ copyright (fbuffer * mac, fbuffer ** buf) {
   int i, macreadx, spaceneeded;
 #ifdef DDEBUG
   if (debuglevel > 0) {
-    fprintf (log_, "copyright:\n input"); wrbuf (mac, 3, 1);
-    fprintf (log_, " output"); wrbuf (*buf, 3, 0); }
+    fprintf (log_, "copyright:\n input"); wrbuf (mac, 3);
+    fprintf (log_, " output"); wrbuf (*buf, 3); }
 #endif
   while (mac != NULL) {
     if ((*buf) == NULL) {
@@ -1073,7 +1051,7 @@ copyright (fbuffer * mac, fbuffer ** buf) {
     }
 #ifdef DDEBUG
   if (debuglevel > 0) {
-    fprintf (log_, " result"); wrbuf (*buf, 3, 0); }
+    fprintf (log_, " result"); wrbuf (*buf, 3); }
 #endif
 }
 
@@ -1177,7 +1155,7 @@ copyargbody (int *parenlevel, fbuffer ** p2) {
     	  }
         }
       if (inputeof) {
-	    markerror (806);
+	    markerror (805);
 	    j = p1->savedlen;
 	    inarg = false;
 	    *parenlevel = -1;
@@ -1256,7 +1234,8 @@ arg
    								old-arg stack or make a new one */
 void
 newarg (arg ** ar) {
-  if (freearg == NULL) { *ar = malloc (sizeof (arg)); }
+  if (freearg == NULL) { *ar = malloc (sizeof (arg));
+    if (*ar==NULL){ fatal(9); } }
   else { *ar = freearg; freearg = freearg->nexta; }
   (*ar)->argbody = NULL;
   (*ar)->highera = NULL;
@@ -1277,7 +1256,7 @@ ismacro (Char * chb, chbufinx obi, chbufinx chbi) {
   int i;
   if (debuglevel > 0) {
     fprintf (log_, "\nismacro[");
-    for (i = obi; i < chbi; i++) { putc (chb[i], log_); }
+    for (i = obi; i < chbi; i++) { wchar (&log_,chb[i]); }
     fprintf (log_, "]:"); }
 #endif
   if (oldsymb == Xdefine) { skipwhite (); }
@@ -1316,7 +1295,12 @@ expandarg (Char * chb, chbufinx chbs, chbufinx chbi) {
   int n = 0;
   int i;
   arg *ar;
-
+#ifdef DDEBUG
+  if (debuglevel > 0) {
+    fprintf(log_, "\nexpandarg(");
+    for (i = chbs; i < chbi; i++) { wchar(&log_,chb[i]); }
+    fprintf(log_, ") "); }
+#endif
   for (i = chbs + 1; i < chbi; i++) { n = (n * 10) + chb[i] - '0'; }
   ar = findarg (args, n);
   backup ();
@@ -1330,6 +1314,10 @@ insertarg (void) {
   int icx;
 
   pushch ();
+#ifdef DDEBUG
+  if (debuglevel > 0) { fprintf(log_, "\n insertarg $"); wchar(&log_, ch);
+    putc('\n', log_); }
+#endif
   if (ch == '+') {
     backup ();
     ch = '$';
@@ -1346,6 +1334,21 @@ insertarg (void) {
     }
 }
 
+boolean
+illegalchar(Char c) {
+#ifdef DDEBUG
+  if (debuglevel > 0) {
+    fprintf(log_, "Marking 800: chr(%d)\"", c);
+    wchar(&log_, c);
+    fprintf(log_, "\" invalid\n");
+    }
+#endif
+  fprintf(errout, "Character chr(%d)\"", c);
+  wchar(&errout, c);
+  fprintf(errout, "\" invalid\n");
+  markerror(800);
+  return false;
+}
 							/* Find the next terminal.
    								Set lexsymb (terminal lexical value),
    								newsymb (terminal number used for parsing),
@@ -1402,7 +1405,7 @@ yylex (attribute * a0) {
       terminalaccepted = false; }
 
     else {
-      firstch = ch;
+      firstch =  ch;
       pushch ();
 							/* Continuation, comment, constant, latex */
       currbuf = inbuf;
@@ -1418,11 +1421,12 @@ yylex (attribute * a0) {
       else if ((firstch == '.') && (currbuf->readx == 3) &&
 		 (inbuf->prevb == NULL) && (newsymb != (-2))) {
 	    readlatex (); }
+/* else if (firstch > ordMAXCH) { terminalaccepted = illegalchar(firstch); } */
 							/* Use the lexical tables to identify
        							terminals, checking for macro names */
       else {
-	    newsymb = entrytv[abs (firstch)];
-	    lxix = entryhp[abs (firstch)];
+	    newsymb = entrytv[firstch];
+	    lxix = entryhp[firstch];
 	    while (lxix != 0) {
 	      if (ch == '$') {
             if (!insertarg ()) { lxix = 0; }
@@ -1503,7 +1507,7 @@ yylex (attribute * a0) {
 	    do { pushch (); } while (isdigit (ch) != 0);
 	    expandarg (chbuf, oldbufi, chbufi);
 	    terminalaccepted = false; }
-      else { markerror (805); } }
+      else { markerror (807); } }
 
     else if (newsymb == Xdo) { skipwhite (); }
 #ifdef DDEBUG
@@ -1530,18 +1534,7 @@ yylex (attribute * a0) {
 	    if (ismacro (chbuf, oldbufi, chbufi)) { terminalaccepted = false; }
         else { newsymb = Xname; } }
 	                        /* char not recognized */
-      else if (newsymb == 0) {
-#ifdef DDEBUG
-	    if (debuglevel > 0) { fprintf (log_,
-		     "\nMarking 800:ord(firstch)=%12d ord(ch)=%12d\n", firstch, ch); }
-#endif
-	    fprintf (errout, "Char chr(%d)", firstch);
-	    putc ('"', errout);
-	    wchar (&errout, firstch);
-	    putc ('"', errout);
-	    fprintf (errout, " unknown\n");
-	    markerror (800);
-	    terminalaccepted = false; }
+      else if (newsymb == 0) { terminalaccepted = illegalchar(firstch); }
       }
     }				/* end of lookahead terminals */
 #ifdef DDEBUG
@@ -1592,7 +1585,7 @@ skiptobrace (void) {
     if (bracelevel <= 0) { break; }
     if (inputeof) {
       bracelevel = 0;
-      if (instring) { markerror (807); } else { markerror (804); } }
+      if (instring) { markerror (806); } else { markerror (804); } }
     else {
       prevch = ch;
       inchar ();
@@ -1619,7 +1612,7 @@ deletebufs (fbuffer ** buf) {
                             /* Get the file name from the command
                                line argument                   */
 void
-P_sun_argv (char *s, int len, int n) {
+P__argv (char *s, int len, int n) {
   char *cp;
   if ((unsigned) n < P_argc) { cp = P_argv[n]; } else { cp = ""; }
   while (*cp && --len >= 0) { *s++ = *cp++; }
@@ -1632,13 +1625,13 @@ openfiles (void) {
   higherinbuf = NULL;
   output = stdout; input = stdin;
   if (argct >= P_argc) { return; }
-  P_sun_argv (infname, sizeof (mstring), argct);
+  P__argv ((char *)infname, sizeof (mstring), argct);
   while (i < j) { if (infname[i-1] != ' ') { i++; } else { j = i; } }
+  infname[i-1] = '\0';
                             /* Open the main input file        */
   if (checkfile (infname, true) != 0) { fatal (1); }
-  if (input != NULL) {
-       input = freopen (trimname (infname, sizeof (mstring)), "r", input); }
-  else { input = fopen (trimname (infname, sizeof (mstring)), "r"); }
+  if (input != NULL) { input = freopen ((char *)infname, "r", input); }
+  else { input = fopen ((char *)infname, "r"); }
   if (input == NULL) { fatal (1); }
 #ifdef DDEBUG
   if (oflag <= 0) { return; }
@@ -1681,7 +1674,7 @@ getoptions (void) {
   argct = 1;
   istop = P_argc;
   while (argct < istop) {
-    P_sun_argv (infname, sizeof (mstring), argct);
+    P__argv ((char *)infname, sizeof (mstring), argct);
     cht = optionchar (infname);
     if (cht == 0) {
       istop = argct;
@@ -1705,6 +1698,7 @@ getoptions (void) {
     else if ((cht == 'h') || (cht == '-')) {
       fprintf (errout, " *** dpic version %s\n",VERSIONDATE);
 #ifdef DDEBUG
+      openlogfile ();
       fprintf (errout, " Debug is enabled\n");
 #endif
       fprintf (errout, " Options:\n");
@@ -1728,6 +1722,9 @@ getoptions (void) {
       fprintf (errout, " *** dpic terminating: Unknown option ");
       if (isprint_ (cht)) { fprintf (errout, "\"%c\"\n", cht); }
       else { fprintf (errout, "\"char(%d)\"\n", cht); }
+#ifdef DDEBUG
+      openlogfile ();
+#endif
       fatal (0);
       }
     argct++;
@@ -1735,7 +1732,7 @@ getoptions (void) {
 }				/* getoptions */
 
 int
-main (int argc, Char * argv[]) {
+main (int argc, char * argv[]) {
   P_argc = argc;
   P_argv = argv;
   redirect = NULL;
@@ -1758,7 +1755,7 @@ main (int argc, Char * argv[]) {
   inputeof = false;
   initrandom ();
 
-  chbuf = malloc (sizeof (chbufarray));
+  chbuf = malloc (sizeof (chbufarray)); if (chbuf==NULL){ fatal(9); }
 #ifdef DDEBUG
   if (debuglevel > 0) { fprintf (log_, "new(chbuf)[%d]\n", odp (chbuf)); }
 #endif
